/*
 * Digimetrie_Sci_Interface.c
 * 
 * Copyright (c) 2011, Digimetrie. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */

 
/* ==================================================================== */
#define __USE_DEPRECATED_STACK_FUNCTIONS__
#include <api_scilab.h>
#include "Scierror.h"
#include <stack-c.h>
#include "sciprint.h"

#include "MALLOC.h"
#include "limits.h"
#include "Digimetrie_Sci.h"

#include <stdio.h>

#ifdef MY_DEBUG
	FILE * pFile;
	#define D_PRINTF( ... ) 							\
		{												\
			pFile = fopen ("d:\\TEMP\\out_sci.txt","a");\
			fprintf( pFile, __VA_ARGS__ );				\
			fclose( pFile );							\
		}
#else
	#define D_PRINTF( ... )
#endif


/* ==================================================================== */
 /** Helper for checking/getting Double args
 \param [in]  fname : function name which ask the operation
 \param [in]  sciErr : scilab error object
 \param [in]  num_arg : num_arg to check
 \param [out,out] pd : will be the value 

 \return 0 if error(s)
*/

int 
P_Get_Arg_As_Double( char* fname, SciErr* sciErr, int num_arg, double *pd )
{
  int *piAddress = NULL;
  short val = 0;

  #ifdef SCI_DEBUG
	sciprint("P_Get_Arg_As_Double() : arg num %i\n", num_arg );
  #endif
  
  /* get Address of inputs */
  *sciErr = getVarAddressFromPosition(pvApiCtx, num_arg, &piAddress);
  if(sciErr->iErr)
  {
    printError(sciErr, 0);
    return 0;
  }
  
  if( isDoubleType(pvApiCtx, piAddress) == 1 )
  {
	#ifdef SCI_DEBUG
	sciprint("P_Get_Arg_As_Double() : is double\n", *pd );
	#endif
	if ( getScalarDouble(pvApiCtx, piAddress, pd) )
		return 0;
  }  
  
  else if( isIntegerType(pvApiCtx, piAddress) == 1 )
  {
	#ifdef SCI_DEBUG
	sciprint("P_Get_Arg_As_Double() : is integer\n", *pd );
	#endif
    if( getScalarInteger16(pvApiCtx, piAddress, &val) )
		return 0;
	*pd = val;
  }
  
  #ifdef SCI_DEBUG
	sciprint("P_Get_Arg_As_Double() : result %f\n", *pd );
  #endif
  
  return 1;
}


/* ==================================================================== */
 /** Helper for checking/getting Integer args
 \see P_Get_Arg_As_Double() for arguments

  \return 0 if error(s)
*/
int Get_Arg_As_Double( char* fname, SciErr* sciErr, int num_arg, double *pd )
{
  int *piAddress = NULL;

  if( P_Get_Arg_As_Double(fname,sciErr,num_arg,pd) == 0 )
  {
	Scierror(999,"%s: Wrong size for input argument #%d: real expected.\n", fname, num_arg);
	return 0;
  }
  
  return 1;
}

/* ==================================================================== */
 /** Helper for checking/getting Integer args
 \see P_Get_Arg_As_Double() for arguments
 \return 0 if error(s)
*/
int Get_Arg_As_Integer( char* fname, SciErr* sciErr, int num_arg, int *pi )
{
  int *piAddress = NULL;
  double dTemp = 0;

  if( P_Get_Arg_As_Double(fname,sciErr,num_arg,&dTemp) == 0 )
  {
	Scierror(999,"%s: Wrong size for input argument #%d: integer expected.\n", fname, num_arg);
	return 0;
  }

  *pi = (int) dTemp;
	
  // Check if it's an integer
  if( dTemp != *pi )
  {
    Scierror(999,"%s: Wrong type for input argument #%d: integer expected.\n", fname, num_arg);
    return 0;
  }

  return 1;
}


/* ==================================================================== */
 /** Helper for checking/getting Integer args
 \see P_Get_Arg_As_Double() for arguments
 \arg [out] arr_d : la matrice applatie
 \arg [out] nb_col : le nombre de colonnes de la matrice
 \arg [out] nb_ligne : le nombre de lignes de la matrice
 \return 0 if error(s)
*/
int
Get_Arg_As_MatrixOfFloat( char* fname, SciErr* sciErr, int num_arg, float** arr_f, int* nb_col, int* nb_ligne )
{
	int *piAddress = NULL;
	int iType, iComplex, i, arr_size;
	double *arr_d;
	
	*arr_f = NULL;
	
	//get variable address of the first input argument
	*sciErr = getVarAddressFromPosition(pvApiCtx, num_arg, &piAddress);
	if(sciErr->iErr)
	{
		printError( sciErr, 0);
		return 0;
	}

	//check type
	*sciErr = getVarType(pvApiCtx, piAddress, &iType);
	if(sciErr->iErr )
	{
		printError( sciErr, 0);
		return 0;
	}
	else if( iType != sci_matrix )
	{
		Scierror(999,"%s: Wrong argument #%d type : matrix of double expected.\n", fname, num_arg);
	}
	
	//get complexity
	iComplex	= isVarComplex(pvApiCtx, piAddress);

	//check complexity
	if(iComplex)
	{
		Scierror(999,"%s: Wrong argument #%d type : matrix of double expected.\n", fname, num_arg);
	}
	else
	{
		//get size and data from Scilab memory
		*sciErr = getMatrixOfDouble(pvApiCtx, piAddress, nb_ligne, nb_col, &arr_d);
	}

	if(sciErr->iErr)
	{
		printError( sciErr, 0);
		return 0;
	}
	
	arr_size = (*nb_ligne) * (*nb_col);
	*arr_f = malloc( sizeof(float) * arr_size );
	for( i =0; i < arr_size; i++ )
		(*arr_f)[i] = (float) arr_d[i];
	
	return 1;
}


/* ==================================================================== */
//! Extrait et vrifie que les 2 premiers arguments passs sont valide en tant que numro de carte et numro de voie
/**
  Return 1 if no error
*/
int
GetCheck__Numbers__Boards_Channel( char* fname, SciErr *sciErr, int* num_carte,  int* num_voie )
{
	/* Cast && Check numro de carte */
	if( ! Get_Arg_As_Integer( fname, sciErr, 1, num_carte ) )
		return 0;

	if( *num_carte < 0 || *num_carte > USHRT_MAX )
	{
		Scierror(999,"%s: Invalid value for input argument #%d: An _positive_ integer expected for a board number.\n", fname, 1);
		return 0;
	}

	/* Cast && Check numro de voie */
	if( ! Get_Arg_As_Integer( fname, sciErr, 2, num_voie ) )
		return 0;

	if( *num_voie < 0 || *num_voie > USHRT_MAX )
	{
		Scierror(999,"%s: Invalid value for input argument #%d: An _positive_ integer expected for a channel number.\n", fname, 2);
		return 0;
	}
	
	return 1;
}

/* ==================================================================== */
int
sci_stack_return_double( double res )
{
	createScalarDouble(pvApiCtx, Rhs + 1, res);
	LhsVar(1) = Rhs + 1; 
	
	return 1;
}

/* ==================================================================== */
int
sci_stack_return_matrick_float( float* arr_res, int nb_col, int nb_ligne )
{
	SciErr sciErr;
	double* arr_cpy;
	int nb_cells, i;
	
	nb_cells = nb_ligne * nb_col;
	arr_cpy = (double*) malloc( nb_cells * sizeof(double) );
	for( i = 0;  i < nb_cells; i++ )
		arr_cpy[i] = arr_res[i];
		
	sciErr = createMatrixOfDouble( pvApiCtx, Rhs + 1, nb_ligne, nb_col, arr_cpy);
	free( arr_cpy );
	
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
	
	LhsVar(1) = Rhs + 1; 
	return 1;
}

/* ==================================================================== */
int
sci_stack_return_int( short res )
{
	createScalarInteger16(pvApiCtx, Rhs + 1, res);
	LhsVar(1) = Rhs + 1; 
	
	return 1;
}


/* ==================================================================== */
int
sci_stack_return_string( const char* sres )
{
	createSingleString(pvApiCtx, Rhs + 1, sres);
	LhsVar(1) = Rhs + 1; 
	
	return 1;
}


/* ==================================================================== */
int sci_digi_AOutf(char *fname)
{
	SciErr sciErr;
	int num_carte, num_voie;
	double vald;
	float val_f;
	int err;

	/* check number of input parameters */
	CheckRhs(3,3) ;

	/* check that we have only 1 parameters output */
	CheckLhs(1,1) ;

	// Verif arguments : num_carte et num_voie
	if( ! GetCheck__Numbers__Boards_Channel( fname, &sciErr, &num_carte, &num_voie ) )
		return 0;
	
	// Verif argument : valeur  fixer pour la voie
	if( ! Get_Arg_As_Double( fname, &sciErr, 3, &vald ) )
		return 0;
	
	val_f = (float) vald;
	
	#ifdef SCI_DEBUG
	sciprint("%s: Valf : %f\n", fname, vald );
	#endif
  
	/* appel de aoutf */
	err = digi_AOutf( num_carte,num_voie,val_f);

	#ifdef SCI_DEBUG
	sciprint("%s: err = %i\n", fname, err );
	#endif

	if( err )
	{
		Scierror(999,"%s: Internal Board error : %s.\n", fname, digi_get_Error_String( err ) );
		return 0;
	}
	
	/* create result on stack */
	sci_stack_return_int( 1 );

	return 0;
}

/* ==================================================================== */
int sci_digi_AOutf_Scan(char *fname)
{
	SciErr sciErr;
	int num_carte, frequence_hz, nb_col, nb_ligne;
	float* arr_val;
	int err;

	//-------------------------------------------------------------------
	//			RECUP/VERIF PARAMETRES
	//-------------------------------------------------------------------
	
	/* check number of input parameters */
	CheckRhs(3,3) ;

	// FIXME : est-ce ok de ne pas vrifier le nombre de valeurs retournes ?
	// check that we have only 1 parameters output
//	CheckLhs(1,1) ;

	// Verif arguments : num_carte
	if( ! Get_Arg_As_Integer( fname, &sciErr, 1, &num_carte ) )
		return 0;

	if( num_carte < 0 || num_carte > USHRT_MAX )
	{
		Scierror(999,"%s: Invalid value for input argument #%d: An _positive_ integer expected for a board number.\n", fname, 1);
		return 0;
	}

	// Verif arguments frequence_hz
	if( ! Get_Arg_As_Integer( fname, &sciErr, 2, &frequence_hz ) )
		return 0;
	
	if( frequence_hz < 0  )
	{
		Scierror(999,"%s: Invalid value for input argument #%d: An _positive_ integer expected for a frequence in hz.\n", fname, 2);
		return 0;
	}
	
	// Verif arguments matrix of double 
	if( ! Get_Arg_As_MatrixOfFloat( fname, &sciErr, 3, &arr_val, &nb_col, &nb_ligne ) )
		return 0;
	
	if( nb_col <= 0 || nb_col >= USHRT_MAX )
	{
		if( arr_val != NULL )
			free( arr_val );
		Scierror(999,"%s: Wrong matrix width for input argument #%d: Number of channel must be a not too big _positive_ integer.\n", fname, 3);
		return 0;
	}
	
	if( nb_ligne <= 0 || nb_ligne >= USHRT_MAX )
	{
		if( arr_val != NULL )
			free( arr_val );
		Scierror(999,"%s: Wrong matrix height for input argument #%d: Number of restitutions must be a not too big _positive_ integer.\n", fname, 3);
		return 0;
	}
	
	
	
	//-------------------------------------------------------------------
	// 					APPEL DE AOUTF_SCAN
	//-------------------------------------------------------------------
	err = digi_AOutf_Scan( num_carte, nb_col, arr_val, nb_ligne, frequence_hz );

	free( arr_val );
	
	#ifdef SCI_DEBUG
	sciprint("%s: err = %i\n", fname, err );
	#endif
	
	if( err )
	{
		free( arr_val );
		Scierror(999,"%s: Internal Board error : %s.\n", fname, digi_get_Error_String( err ) );
		return 0;
	}
	
	// create result on stack
	sci_stack_return_int( 1 );

	return 0;
}


/* ==================================================================== */
int
sci_digi_AInf(char *fname)
{
	SciErr sciErr;
	int num_carte, num_voie,gain;
	float fres;
	int err;

	/* check number of input parameters */
	CheckRhs(3,3) ;

	// check that we have only 1 parameters output
	CheckLhs(1,1) ;

	// Verif arguments : num_carte et num_voie
	if( ! GetCheck__Numbers__Boards_Channel( fname, &sciErr, &num_carte, &num_voie ) )
		return 0;
	
	// Verif arguments gain
	if( ! Get_Arg_As_Integer( fname, &sciErr, 3, &gain ) )
		return 0;
		
	switch( gain )
	{
		case 1:
			gain = 0;
			break;
		case 2:
			gain = 1;
			break;
		case 4:
			gain = 2;
			break;
		case 8:
			gain = 3;
			break;
		default:
			Scierror(999,"%s: Gain invalide \"%i\" : doit tre parmi [1;2;4;8]\n", fname, gain );
			return 0;
	}
	
	// appel de ainf
	err = digi_AInf( num_carte,num_voie,gain,&fres);

	#ifdef SCI_DEBUG
	sciprint("%s: err = %i\n", fname, err );
	#endif
	
	if( err )
	{
		Scierror(999,"%s: Internal Board error : %s.\n", fname, digi_get_Error_String( err ) );
		return 0;
	}
	
	// create result on stack
	sci_stack_return_double( fres );

	return 0;
}
	


/* ==================================================================== */
int
sci_digi_AInf_Scan(char *fname)
{
	SciErr sciErr;
	int num_carte, nb_voies, gain_unique, frequence_hz, nb_acquisitions;
	int i;
	float* f_res;
	USHORT arr_gain[16];
	int err;

	//-------------------------------------------------------------------
	//			RECUP/VERIF PARAMETRES
	//-------------------------------------------------------------------
	
	/* check number of input parameters */
	CheckRhs(5,5) ;

	// FIXME : est-ce ok de ne pas vrifier le nombre de valeurs retournes ?
	// check that we have only 1 parameters output
//	CheckLhs(1,1) ;

	// Verif arguments : num_carte
	if( ! Get_Arg_As_Integer( fname, &sciErr, 1, &num_carte ) )
		return 0;

	if( num_carte < 0 || num_carte > USHRT_MAX )
	{
		Scierror(999,"%s: Invalid value for input argument #%d: An _positive_ integer expected for a board number.\n", fname, 1);
		return 0;
	}

	/* Cast && Check nb_voies */
	if( ! Get_Arg_As_Integer( fname, &sciErr, 2, &nb_voies ) )
		return 0;

	if( nb_voies <= 0 || nb_voies > USHRT_MAX )
	{
		Scierror(999,"%s: Invalid value for input argument #%d: An _positive_ integer expected for the number of channels.\n", fname, 2);
		return 0;
	}
	
	// Verif arguments gain_unique
	if( ! Get_Arg_As_Integer( fname, &sciErr, 3, &gain_unique ) )
		return 0;
	
	switch( gain_unique )
	{
		case 1:
			gain_unique = 0;
			break;
		case 2:
			gain_unique = 1;
			break;
		case 4:
			gain_unique = 2;
			break;
		case 8:
			gain_unique = 3;
			break;
		default:
			Scierror(999,"%s: Invalid value for input argument #%d: Invalid Gain \"%i\" : must be among [1;2;4;8]\n", fname, 3, gain_unique );
			return 0;
	}
	
	// Verif arguments frequence_hz
	if( ! Get_Arg_As_Integer( fname, &sciErr, 4, &frequence_hz ) )
		return 0;
	
	if( frequence_hz < 0  )
	{
		Scierror(999,"%s: Invalid value for input argument #%d: An _positive_ integer expected for a frequence in hz.\n", fname, 4);
		return 0;
	}
	
	// Verif arguments nb_acquisitions
	if( ! Get_Arg_As_Integer( fname, &sciErr, 5, &nb_acquisitions ) )
		return 0;
	
	if( nb_acquisitions <= 0 || nb_acquisitions > USHRT_MAX )
	{
		Scierror(999,"%s: Invalid value for input argument #%d: A not too big _positive_ integer expected for a quantity of acquisitions.\n", fname, 5);
		return 0;
	}
	
	
	
	
	//-------------------------------------------------------------------
	// 					APPEL DE AINF_SCAN
	//-------------------------------------------------------------------
	f_res = (float*) malloc( nb_acquisitions * nb_voies * sizeof(float) );
	for( i = 0; i < 16; i++ )
		arr_gain[i] = gain_unique;

	err = digi_AInf_Scan( num_carte, nb_voies, arr_gain, f_res, frequence_hz, nb_acquisitions );

	#ifdef SCI_DEBUG
	sciprint("%s: err = %i\n", fname, err );
	#endif
	
	if( err )
	{
		free( f_res );
		Scierror(999,"%s: Internal Board error : %s.\n", fname, digi_get_Error_String( err ) );
		return 0;
	}
	
	// create result on stack
	sci_stack_return_matrick_float( f_res, nb_voies, nb_acquisitions );
	free( f_res );

	return 0;
}



/* ==================================================================== */
int sci_digi_DOut(char *fname)
{
	SciErr sciErr;
	int num_carte, num_voie;
	int ival;
	int err = 0;
	
	/* check number of input parameters */
	CheckRhs(3,3) ;

	/* check that we have only 1 parameters output */
	CheckLhs(1,1) ;

	// Verif arguments : num_carte et num_voie
	if( ! GetCheck__Numbers__Boards_Channel(fname, &sciErr, &num_carte, &num_voie ) )
		return 0;
	
	// Verif argument : valeur  fixer pour la voie
	if( ! Get_Arg_As_Integer( fname, &sciErr, 3, &ival ) )
		return 0;
	
	/* appel de dout */
//	ival = 1000 * num_carte + 100 * num_voie + ival;
	err = digi_DOut( num_carte, num_voie, ival);

	if( err )
	{
		Scierror(999,"%s: Internal Board error : %s.\n", fname, digi_get_Error_String( err ) );
		return 0;
	}
	
	/* create result on stack */
	sci_stack_return_int( 1 );

	return 0;
}
	
/* ==================================================================== */
int 
sci_digi_BOut(char *fname)
{
	SciErr sciErr;
	int num_carte, num_voie;
	int num_ligne, val_ligne;
	int err = 0;
	
	/* check number of input parameters */
	CheckRhs(4,4) ;

	/* check that we have only 1 parameters output */
	CheckLhs(1,1) ;

	// Verif arguments : num_carte et num_voie
	if( ! GetCheck__Numbers__Boards_Channel(fname, &sciErr, &num_carte, &num_voie ) )
		return 0;
	
	// Verif argument : numero de la ligne
	if( ! Get_Arg_As_Integer( fname, &sciErr, 3, &num_ligne ) )
		return 0;
	
	// Verif argument : valeur de la ligne
	if( ! Get_Arg_As_Integer( fname, &sciErr, 4, &val_ligne ) )
		return 0;
	
	
	/* appel de bout */
	err = digi_BOut( num_carte, num_voie, num_ligne, val_ligne );

	if( err )
	{
		Scierror(999,"%s: Internal Board error : %s.\n", fname, digi_get_Error_String( err ) );
		return 0;
	}
	
	/* create result on stack */
	sci_stack_return_int( 1 );

	return 0;
}

	
/* ==================================================================== */
int sci_digi_DIn(char *fname)
{
	SciErr sciErr;
	int num_carte, num_voie;
	int ires;
	int err;
	
	/* check number of input parameters */
	CheckRhs(2,2) ;

	// check that we have only 1 parameters output
	CheckLhs(1,1) ;

	// Verif arguments : num_carte et num_voie
	if( ! GetCheck__Numbers__Boards_Channel( fname, &sciErr, &num_carte, &num_voie ) )
		return 0;
	
	// appel de din
	err = digi_DIn(  num_carte, num_voie, &ires);

	if( err )
	{
		Scierror(999,"%s: Internal Board error : %s.\n", fname, digi_get_Error_String( err ) );
		return 0;
	}
	
	// create result on stack
	sci_stack_return_int( ires );

	return 0;
}

/* ==================================================================== */
int sci_digi_BIn(char *fname)
{
	SciErr sciErr;
	int num_carte, num_voie, num_ligne;
	int ires;
	int err;
	
	/* check number of input parameters */
	CheckRhs(3,3) ;

	// check that we have only 1 parameters output
	CheckLhs(1,1);

	// Verif arguments : num_carte et num_voie
	if( ! GetCheck__Numbers__Boards_Channel( fname, &sciErr, &num_carte, &num_voie ) )
		return 0;
	
	// Verif argument : numero de la ligne
	if( ! Get_Arg_As_Integer( fname, &sciErr, 3, &num_ligne ) )
		return 0;
	
	// appel de bin
	err = digi_BIn(  num_carte, num_voie, num_ligne, &ires);

	if( err )
	{
		Scierror(999,"%s: Internal Board error : %s.\n", fname, digi_get_Error_String( err ) );
		return 0;
	}
	
	// create result on stack
	sci_stack_return_int( ires );

	return 0;
}

/* ==================================================================== */
// Fonction multi-usages pour tester|parametrer un carte Digimtrie
// Arguments :
//   0 -> ENTIER : Numero de la carte
//   1 -> ENTIER : Numero de la voie
//   2 -> ENTIER : Type de test
//
//  Si "Type de test" :
//		== 0 : Test que la carte existe
//		== 1 : Test la voie spcifie en AIN
//		== 2 : Test la voie spcifie en AOUT
//		== 3 : Test la voie spcifie en DIN
//		== 4 : Test la voie spcifie en DOUT
//
// Retourne :
//			0 si tout est ok
//			sinon le numro de l'erreur Digimtrie
//
int
sci_digi_TestParam(char *fname)
{
	SciErr sciErr;
	int num_carte, num_voie, typeTest;
	int ires;
	
	/* check number of input parameters */
	CheckRhs(3,3) ;

	// check that we have only 1 parameters output
	CheckLhs(1,1) ;

	// Verif arguments : num_carte et num_voie
	if( ! GetCheck__Numbers__Boards_Channel( fname, &sciErr, &num_carte, &num_voie ) )
		return 0;
	
	// Verif arguments typeTest
	if( ! Get_Arg_As_Integer( fname, &sciErr, 3, &typeTest ) )
		return 0;
	
//  	Scierror(999,"sci_digi_TestParam : %i, %i, %i\n", num_carte, num_voie, typeTest);
	ires = digi_TestParam( num_carte, num_voie, typeTest );
	
	// create result on stack
	sci_stack_return_int( ires );

	return 0;
}


/* ==================================================================== */
int
sci_digi_get_Error_String( char* fname )
{
	SciErr sciErr;
	int num_err;
	const char* sres;

	/* check number of input parameters */
	CheckRhs(1,1);

	// check that we have only 1 parameters output
	CheckLhs(1,1) ;
	
	// Verif arguments num_err
	if( ! Get_Arg_As_Integer( fname, &sciErr, 1, &num_err ) )
		return 0;
	
	sres = digi_get_Error_String( num_err );
	
	// create result on stack
	sci_stack_return_string( sres );
	
	return 0;
}

/* ==================================================================== */
int
sci_digi_Reinit( char* fname )
{
	int err_or_nb_cartes;
	const char* sres;

	D_PRINTF( "sci_digi_Reinit() : START\n" );
	
	/* check number of input parameters */
	CheckRhs(0,0);

	// check that we have only 1 parameters output
	CheckLhs(1,1) ;
	
	D_PRINTF( "sci_digi_Reinit() : Arguments Checks finish\n" );
	sciprint("Reinitialising Digimetrie's boards...\n" );
	
	err_or_nb_cartes = digi_Reinit();
	
	D_PRINTF( "digi_Reinit()  result %i\n", err_or_nb_cartes);
	
	if( err_or_nb_cartes < 0 )
	{	
		sres = digi_get_Error_String( - err_or_nb_cartes );
		
		D_PRINTF( "digi_Reinit()  error %s\n", sres);
		sciprint("FAILED to Reinitialising Digimetrie's boards : %s\n", sres );
		
		// create result on stack
		D_PRINTF( "digi_Reinit() : return val 0\n", sres);
		sci_stack_return_double( 0 );
		
		D_PRINTF( "digi_Reinit() : END\n", sres);
		return 0;
	}
	
	D_PRINTF( "digi_Reinit() : %i boards found\n", err_or_nb_cartes);
	sciprint("... %i boards found in the system\n", err_or_nb_cartes );
	
	D_PRINTF( "digi_Reinit() : return val 1\n", sres);
	sci_stack_return_double( 1 );
	
	D_PRINTF( "digi_Reinit() : END\n", sres);
	return 0;
}

