/*
 * DigiTool.c
 * 
 * Copyright (c) 2011, Digimetrie. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */

#define TEST_MACROS

#ifndef TEST_MACROS

#include <windows.h>
#include <winnt.h>

#endif // TEST_MACROS

#include "DigiTool.h"

//-----------------------------------------------------------------------------------------
const char*
Digi_GetErrorString( int num_err )
{
	switch( num_err )
	{
		#define D_DIGI_ERR( ID_ERR, NUM_ERR, STR_ERR )			\
		case NUM_ERR:					\
			return STR_ERR;
		#include "DigiDefs_Errors.def.h"
	}

	return "ERREUR Non Listee dans Digitool.h";
}


//-------------------------------------------
//! Retourne le nom de la carte correspondant
const char*
Digi_GetCarte_Nom( int id_carte )
{
	switch( id_carte )
	{
		#define D_CARTE_DIGI( ID, NUM,	 S_NOM, S_DLL )	\
		case ID:				\
			return S_NOM;

		#include "DigiDefs_Cartes.def.h"
	}

	return "ID de carte INCONNU";
}

//-------------------------------------------
const char*
Digi_GetCarte_Dll( int id_carte )
{
	switch( id_carte )
	{
		#define D_CARTE_DIGI( ID, NUM,	 S_NOM, S_DLL )	\
		case ID:				\
			return S_DLL;

		#include "DigiDefs_Cartes.def.h"
	}

	return "";
}

//-------------------------------------------
const char*
Digi_GetEventString( int id_event )
{
	switch( id_event )
	{
		#define X_DIGI_EVT( ID_EVT, NUM_EVT, STR_DESCRIPTIVE )	\
		case ID_EVT:				\
			return STR_DESCRIPTIVE;

		#include "DigiDefs_Events.def.h"
	}

	return "";
}

//***************************************************************
//***************************************************************
//***************************************************************
//***************************************************************

//------------------------
// Listage des fonctions prises en compte par dans le chargement automatique "Digi_Init_Functions_Pointers"
#define XMACRO_API_FUNCTIONS()				\
		X_API_FCT( Get_Conf )				\
		X_API_FCT( Init_Capt )				\
		X_API_FCT( Reset_carte )			\
		X_API_FCT( Dinit )					\
		X_API_FCT( Din )					\
		X_API_FCT( Dout )					\
		X_API_FCT( Dinscan )				\
		X_API_FCT( Doutscan )				\
		X_API_FCT( BOut )					\
		X_API_FCT( BIn )					\
		X_API_FCT( Ain )					\
		X_API_FCT( Ainv )					\
		X_API_FCT( Ainf )					\
		X_API_FCT( Ainfv )					\
		X_API_FCT( Ainscan )				\
		X_API_FCT( Ainscan_buff )			\
		X_API_FCT( Ainfscan )				\
		X_API_FCT( Ainfscan_buff )			\
		X_API_FCT( Aout )					\
		X_API_FCT( Aoutf )					\
		X_API_FCT( Aoutscan )				\
		X_API_FCT( Aoutfscan )				\
		X_API_FCT( Ginit )					\
		X_API_FCT( Gstart )					\
		X_API_FCT( Gstop )					\
		X_API_FCT( Cinit )					\
		X_API_FCT( Cstart )					\
		X_API_FCT( Cstop )					\
		X_API_FCT( Cread )					\
		X_API_FCT( Cread32 )				\
		X_API_FCT( Gs_init )				\
		X_API_FCT( Gs_stop )				\
		X_API_FCT( Multiscan )				\
		X_API_FCT( Multiscan_buff )			\
		X_API_FCT( Test_buff )				\
		X_API_FCT( Write_Pot )				\
		X_API_FCT( Read_Pot )				\
		X_API_FCT( Calibrage_An )			\
		X_API_FCT( Tst_trigger )			\
		X_API_FCT( Get_id )					\
		X_API_FCT( Set_id )					\
		X_API_FCT( User_WriteEep )			\
		X_API_FCT( User_ReadEep )			\
		X_API_FCT( Capt_WriteEep )			\
		X_API_FCT( Capt_ReadEep )			\
		X_API_FCT( Get_devices )			\
		X_API_FCT( GetInfo_Reseau )			\
		X_API_FCT( Init_Reseau )			\
		X_API_FCT( Stop_wdg )				\
		X_API_FCT( Target_rev )				\
		X_API_FCT( Digitool_rev )			\
		X_API_FCT( GetDeviceNumber )		\
		X_API_FCT( GetPleineEchelleAD )		\
		X_API_FCT( SetPleineEchelleAD )		\
		X_API_FCT( GetPleineEchelleDA )		\
		X_API_FCT( SetPleineEchelleDA )		\
		X_API_FCT( SetTriggerAD )			\
		X_API_FCT( GetTriggerAD )			\
		X_API_FCT( StopScan )				\
		X_API_FCT( EventCallBack__Set )		\
		X_API_FCT( EventCallBack__Get )		\
		X_API_FCT( Set_Verrouillage_Type )	\
		X_API_FCT( Get_last_error )			\
		X_API_FCT( Get_Nb_Cartes )			\
		X_API_FCT( Multifscan_buff )		\
		X_API_FCT(Multifscan )				\


//------------------------
// Helpers
// Dclaration des pointeurs de fonctions
#define X_API_FCT( F_NAME )				\
	PROTO_##F_NAME pf_##F_NAME = NULL;	\

XMACRO_API_FUNCTIONS()

#undef X_API_FCT

// Enumeration des noms de fonctions (pour savoir le nombre de fonctions
#define X_API_FCT( F_NAME )	\
	E_##F_NAME,					\

enum E_Api_Function_Names
{
	XMACRO_API_FUNCTIONS()

	nb_api_functions
};
#undef X_API_FCT

// Tableau des noms de fonctions
#define X_API_FCT( F_NAME )		\
	#F_NAME,		\

const char* arr_api_fcts_names[] = {
	XMACRO_API_FUNCTIONS()
	"_END_"
	};
#undef X_API_FCT

// Tableau des pointeurs de pointeurs de fonctions
#define X_API_FCT( F_NAME )		\
	(FARPROC*) &pf_##F_NAME ,	\

FARPROC* arr_api_fcts_ptrs[] = {
	XMACRO_API_FUNCTIONS()
	NULL
	};
#undef X_API_FCT


//--------------------------------
static 
void
reset_api_pointeurs()
{
	size_t i;
	for( i = 0; i < nb_api_functions; i++ )
		*arr_api_fcts_ptrs[i] = NULL;
}

//--------------------------------
static 
BOOL
assign_api_pointeur( HMODULE dllHandle, const char* origin_dll_f_name )
{
	size_t i, dll_f_name__len, f_name__len;
	const char *f_name, *dll_f_name;

	dll_f_name = origin_dll_f_name;
	dll_f_name__len = strlen( dll_f_name );
	if( dll_f_name__len == 0)
		return FALSE;
	UNPREFIXE_DLL_FCT_NAME( dll_f_name, dll_f_name__len )

	for( i = 0; i < nb_api_functions; i++ )
	{
		f_name = arr_api_fcts_names[i];
		f_name__len = strlen( f_name );

		if( dll_f_name__len < f_name__len )
			continue;
		if( dll_f_name[f_name__len] != '@' &&  dll_f_name[f_name__len] != '\0' )
			continue;

		if( strncmp( dll_f_name, f_name, f_name__len ) == 0 )
		{
			*arr_api_fcts_ptrs[i] =  GetProcAddress(dllHandle, origin_dll_f_name );
			return TRUE;
		}
	}
	return FALSE;
}

//-------------------------------------------
BOOL
Digi_Init_Functions_Pointers( HMODULE h_loaded_Dll, P_STR dll_path )
{
	BYTE *hMod;
	IMAGE_NT_HEADERS *pnt;
	IMAGE_EXPORT_DIRECTORY *exp;
	DWORD *dwFunctions, ctr;
	BOOL res;

	res = FALSE;

	// Reset des pointeurs 
	reset_api_pointeurs();

	// Si aucune Dll charge
	if( h_loaded_Dll == NULL )
		return FALSE;

	// Rcuprations des noms de fonctions dans la Dll
	hMod = (BYTE*)GetModuleHandle(dll_path);
	pnt = (IMAGE_NT_HEADERS*)&hMod[ ((PIMAGE_DOS_HEADER)hMod)->e_lfanew];
	exp = (IMAGE_EXPORT_DIRECTORY*)&hMod[pnt->OptionalHeader.DataDirectory->VirtualAddress];
	dwFunctions = (DWORD*)&hMod[exp->AddressOfNames];

	// Allocation des pointeurs de la Dll
	for ( ctr = 0; ctr < exp->NumberOfNames; ctr++)
	{
		res = ( assign_api_pointeur( h_loaded_Dll,  (char*)&hMod[dwFunctions[ctr]] )) || res;
	}

	return res;
}
