/*
 * DigiTool_PciLab.c
 * 
 * Copyright (c) 2011, Digimetrie. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */

#include <windows.h>
#include <winnt.h>

#include "DigiTool_PciOpto.h"


//------------------------
// Listage des fonctions prises en compte par dans le chargement automatique "Digi_Init_Functions_Pointers"
#define XMACRO_API_FUNCTIONS()				\
		X_API_FCT( PCIOPTO_DI_Channel )		\
		X_API_FCT( PCIOPTO_DO_Channel )		\
		X_API_FCT( PCIOPTO_Initial )		\
		X_API_FCT( PCIOPTO_Switch_Card )	\
		X_API_FCT( PCIOPTO_INT_Start )		\
		X_API_FCT( PCIOPTO_INT_Stop )		\
		


//------------------------
// Helpers
// Dclaration des pointeurs de fonctions
#define X_API_FCT( F_NAME )			\
	PROTO_##F_NAME pf_##F_NAME;		\

XMACRO_API_FUNCTIONS()

#undef X_API_FCT

// Enumeration des noms de fonctions (pour savoir le nombre de fonctions
#define X_API_FCT( F_NAME )	\
	E_##F_NAME,					\

enum E_Api_Function_Names__PCILAB
{
	XMACRO_API_FUNCTIONS()

	nb_api_functions
};
#undef X_API_FCT

// Tableau des noms de fonctions
#define X_API_FCT( F_NAME )		\
	#F_NAME ,		\

const char* arr_api_fcts_names__PCILAB[] = {
	XMACRO_API_FUNCTIONS()
	"_END_"
	};
#undef X_API_FCT

// Tableau des pointeurs de pointeurs de fonctions
#define X_API_FCT( F_NAME )		\
	(FARPROC*) &pf_##F_NAME ,	\

FARPROC* arr_api_fcts_ptrs__PCILAB[] = {
	XMACRO_API_FUNCTIONS()
	NULL
	};
#undef X_API_FCT


//--------------------------------
static 
void
reset_api_pointeurs()
{
	size_t i;
	for( i = 0; i < nb_api_functions; i++ )
		*arr_api_fcts_ptrs__PCILAB[i] = NULL;
}

//--------------------------------
static 
BOOL
assign_api_pointeur( HMODULE dllHandle, const char* origin_dll_f_name )
{
	size_t i, dll_f_name__len, f_name__len;
	const char *f_name, *dll_f_name;

	dll_f_name = origin_dll_f_name;
	dll_f_name__len = strlen( dll_f_name );
	if( dll_f_name__len == 0)
		return FALSE;
	UNPREFIXE_DLL_FCT_NAME( dll_f_name, dll_f_name__len )

	for( i = 0; i < nb_api_functions; i++ )
	{
		f_name = arr_api_fcts_names__PCILAB[i];
		f_name__len = strlen( f_name );

		if( dll_f_name__len < f_name__len )
			continue;
		if( dll_f_name[f_name__len] != '@' &&  dll_f_name[f_name__len] != '\0' )
			continue;

		if( strncmp( dll_f_name, f_name, f_name__len ) == 0 )
		{
			*arr_api_fcts_ptrs__PCILAB[i] =  GetProcAddress(dllHandle, origin_dll_f_name );
			return TRUE;
		}
	}
	return FALSE;
}


//-------------------------------------------
BOOL
Digi_Init_Functions_Pointers__PCIOPTO( HMODULE h_loaded_Dll, P_STR dll_path )
{
	BYTE *hMod;
	IMAGE_NT_HEADERS *pnt;
	IMAGE_EXPORT_DIRECTORY *exp;
	DWORD *dwFunctions, ctr;
	BOOL res;

	res = FALSE;

	// Reset des pointeurs 
	reset_api_pointeurs();

	// Si aucune Dll charge
	if( h_loaded_Dll == NULL )
		return FALSE;

	// Rcuprations des noms de fonctions dans la Dll
	hMod = (BYTE*)GetModuleHandle(dll_path);
	pnt = (IMAGE_NT_HEADERS*)&hMod[ ((PIMAGE_DOS_HEADER)hMod)->e_lfanew];
	exp = (IMAGE_EXPORT_DIRECTORY*)&hMod[pnt->OptionalHeader.DataDirectory->VirtualAddress];
	dwFunctions = (DWORD*)&hMod[exp->AddressOfNames];

	// Allocation des pointeurs de la Dll
	for ( ctr = 0; ctr < exp->NumberOfNames; ctr++)
	{
		res = ( assign_api_pointeur( h_loaded_Dll,  (char*)&hMod[dwFunctions[ctr]] )) || res;
	}

	return res;
}
